/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2012 by Tidalwave s.a.s. (http://tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.integritychecker.ui.impl;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.NotThreadSafe;
import java.util.concurrent.TimeUnit;
import org.joda.time.Duration;
import org.joda.time.format.PeriodFormat;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.actor.MessageSupport;
import it.tidalwave.actor.Collaboration;
import it.tidalwave.actor.CollaborationCompletedMessage;
import it.tidalwave.actor.CollaborationStartedMessage;
import it.tidalwave.actor.annotation.Actor;
import it.tidalwave.actor.annotation.ListensTo;
import it.tidalwave.actor.annotation.Message;
import it.tidalwave.actor.annotation.OriginatedBy;
import it.tidalwave.actor.io.filescan.FileDiscoveredMessage;
import it.tidalwave.actor.io.filescan.FileScanRequestMessage;
import it.tidalwave.integritychecker.fingerprint.FingerprintComputedMessage;
import it.tidalwave.integritychecker.ui.IntegrityCheckerPresentation;
import it.tidalwave.integritychecker.ui.IntegrityCheckerPresentation.Statistics;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

/***********************************************************************************************************************
 * 
 * @stereotype Actor
 * @stereotype Controller
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Actor(threadSafe=false) @NotThreadSafe @Slf4j
public class IntegrityCheckerPresentationControllerActor
  {
    private static final double K10 = 1000;
    private static final double M10 = 1000000;
    
    @Message(outOfBand=true, daemon=true) @ToString
    private static class RefreshPresentationMessage extends MessageSupport
      {
      }
            
    private final IntegrityCheckerPresentationBuilder presentationBuilder = Locator.find(IntegrityCheckerPresentationBuilder.class);
    
    private final IntegrityCheckerPresentation presentation = presentationBuilder.createPresentation();
    
    private final Statistics statistics = new Statistics();
    
    private boolean refreshing = false;
    
    private int totalFileCount;
    
    private long totalDataSize;
    
    private int processedFileCount;
    
    private long processedDataSize;
    
    /*******************************************************************************************************************
     * 
     * Resets statistics when a new scan is started.
     * 
     ******************************************************************************************************************/
    public void onScanStarted (final @ListensTo @Nonnull CollaborationStartedMessage cMessage,
                               final @OriginatedBy @Nonnull FileScanRequestMessage message)
      {
        log.info("onScanStarted({})", message);
        totalFileCount = 0;
        totalDataSize = 0;
        processedFileCount = 0;
        processedDataSize = 0;
        refreshing = true;
        new RefreshPresentationMessage().send();
      }
    
    /*******************************************************************************************************************
     * 
     * Stops activities when the scan has been completed.
     * 
     ******************************************************************************************************************/
    public void onScanCompleted (final @ListensTo @Nonnull CollaborationCompletedMessage cMessage,
                                 final @OriginatedBy @Nonnull FileScanRequestMessage message)
      {
        log.info("onScanCompleted({})", message);
        refreshing = false;
        statistics.setProgressLabel("done");
        new RefreshPresentationMessage().send();
      }
    
    /*******************************************************************************************************************
     * 
     * Updates the statistics for a new discovered file.
     * 
     ******************************************************************************************************************/
    public void onFileDiscovered (final @ListensTo @Nonnull FileDiscoveredMessage message)
      {
        log.trace("onFileDiscovered({})", message);
        totalFileCount++;
        totalDataSize += message.getPath().toFile().length();
      }
    
    /*******************************************************************************************************************
     * 
     * Updates the statistics for a new fingerprint computed.
     * 
     ******************************************************************************************************************/
    public void onFingerprintComputed (final @ListensTo @Nonnull FingerprintComputedMessage message)
      {
        log.trace("onFingerprintComputed({})", message);
        processedFileCount++;
        processedDataSize += message.getPath().toFile().length();
        final Collaboration collaboration = message.getCollaboration();
        final long elapsedTime = collaboration.getDuration().getMillis();
        final double speed = (processedDataSize / M10) / (elapsedTime / K10);
        final int eta = (int)(((totalDataSize - processedDataSize) / M10) / speed);
        
        statistics.setProcessed(String.format("%.1f MB in %d files", processedDataSize / M10, processedFileCount));
        statistics.setTotal(String.format("%.1f MB in %d files", totalDataSize / M10, totalFileCount));
        statistics.setElapsedTime(PeriodFormat.getDefault().print(new Duration(elapsedTime - elapsedTime % 1000).toPeriod()));
        statistics.setRemainingTime(PeriodFormat.getDefault().print(new Duration(eta * 1000).toPeriod()));
        statistics.setSpeed(String.format("%.1f MB/s", speed));
        statistics.setPending("" + collaboration.getDeliveringMessagesCount());
        statistics.setRunning("" + collaboration.getRunningThreadsCount());
        statistics.setProgress(processedDataSize / (float)totalDataSize);
        statistics.setProgressLabel(String.format("%.1f %%", 100 * statistics.getProgress()));
      }
    
    /*******************************************************************************************************************
     * 
     * Updates the view.
     * 
     ******************************************************************************************************************/
    private void updatePresentation (final @ListensTo @Nonnull RefreshPresentationMessage message)
      {
        presentation.updateStatistics(statistics);
        
        if (refreshing)
          {
            new RefreshPresentationMessage().sendLater(1, TimeUnit.SECONDS);
          }
      }
  }
